# 使用local volume存储ES数据

## 主机规划
    master1   192.168.206.120
    node1     192.168.206.121
    node2     192.168.206.122
    
## 创建es存储数据目录,并授权
    chown -R 1000:1000 /data/local/vol1  #先cat /etc/group 查看是否有uid=1000的用户,没有则创建 

## 创建静态PV
    apiVersion: v1
    kind: PersistentVolume
    metadata:
      name: local-pv-1
    spec:
      capacity:
        storage: 5Gi
      accessModes:
      - ReadWriteOnce
      persistentVolumeReclaimPolicy: Retain
      storageClassName: local-volume
      local:
        path: /data/local/vol1
      nodeAffinity:
        required:
          nodeSelectorTerms:
          - matchExpressions:
            - key: kubernetes.io/hostname
              operator: In
              values:
              - node1
    
    ---
    apiVersion: v1
    kind: PersistentVolume
    metadata:
      name: local-pv-2
    spec:
      capacity:
        storage: 5Gi
      accessModes:
      - ReadWriteOnce
      persistentVolumeReclaimPolicy: Retain
      storageClassName: local-volume
      local:
        path: /data/local/vol1
      nodeAffinity:
        required:
          nodeSelectorTerms:
          - matchExpressions:
            - key: kubernetes.io/hostname
              operator: In
              values:
              - master1

    ---
    apiVersion: v1
    kind: PersistentVolume
    metadata:
      name: local-pv-3
    spec:
      capacity:
        storage: 5Gi
      accessModes:
      - ReadWriteOnce
      persistentVolumeReclaimPolicy: Retain
      storageClassName: local-volume
      local:
        path: /data/local/vol1
      nodeAffinity:
        required:
          nodeSelectorTerms:
          - matchExpressions:
            - key: kubernetes.io/hostname
              operator: In
              values:
              - node2
              
## 创建ES
    apiVersion: v1
    kind: ConfigMap
    metadata:
      name: es
    data:
      elasticsearch.yml: |
        cluster.name: "${NAMESPACE}"
        node.name: "${POD_NAME}"
        network.host: 0.0.0.0
        discovery.seed_hosts: "es-in"
        cluster.initial_master_nodes: "es-0,es-1,es-2"
        node.master: "true"
        node.data: "true"
    ---
    apiVersion: v1
    kind: Service
    metadata:
      name: es-in
      labels:
        app: es
    spec:
      clusterIP: None
      ports:
        - port: 9300
          protocol: TCP
          name: in
      selector:
        app: es
    ---
    apiVersion: v1
    kind: Service
    metadata:
      name: es-out
      labels:
        app: es
    spec:
      ports:
        - port: 9200
          protocol: TCP
          name: out
      selector:
        app: es
    ---
    apiVersion: apps/v1
    kind: StatefulSet
    metadata:
      name: es
    spec:
      serviceName: es
      selector:
        matchLabels:
          app: es
      replicas: 3
      template:
        metadata:
          labels:
            app: es
        spec:
          terminationGracePeriodSeconds: 10
          containers:
            - name: es
              image: elasticsearch:7.5.0
              imagePullPolicy: Always
              ports:
                - containerPort: 9200
                  protocol: TCP
                - containerPort: 9300
                  protocol: TCP
              env:
                - name: NAMESPACE
                  valueFrom:
                    fieldRef:
                      fieldPath: metadata.namespace
                - name: POD_NAME
                  valueFrom:
                    fieldRef:
                      fieldPath: metadata.name
                - name: ES_JAVA_OPTS
                  value: "-Xmx1g -Xms1g"
              volumeMounts:
                - mountPath: /usr/share/elasticsearch/data
                  name: es-data
                - name: es-config
                  mountPath: /usr/share/elasticsearch/config/elasticsearch.yml
                  subPath: elasticsearch.yml
          volumes:
            - name: es-config
              configMap:
                name: es
      volumeClaimTemplates:
      - metadata:
          name: es-data
        spec:
          accessModes: [ "ReadWriteOnce" ]
          storageClassName: "local-volume"
          resources:
            requests:
              storage: 5Gi

## 验证
    [root@master1 es]# kubectl get svc
    NAME         TYPE        CLUSTER-IP     EXTERNAL-IP   PORT(S)    AGE
    es-in        ClusterIP   None           <none>        9300/TCP   30m
    es-out       ClusterIP   10.96.58.123   <none>        9200/TCP   30m
    kubernetes   ClusterIP   10.96.0.1      <none>        443/TCP    46h
    [root@master1 es]# curl http://10.96.58.123:9200
    {
      "name" : "es-2",
      "cluster_name" : "default",
      "cluster_uuid" : "dcauj-6ESlahu273KOT94w",
      "version" : {
        "number" : "7.5.0",
        "build_flavor" : "default",
        "build_type" : "docker",
        "build_hash" : "e9ccaed468e2fac2275a3761849cbee64b39519f",
        "build_date" : "2019-11-26T01:06:52.518245Z",
        "build_snapshot" : false,
        "lucene_version" : "8.3.0",
        "minimum_wire_compatibility_version" : "6.8.0",
        "minimum_index_compatibility_version" : "6.0.0-beta1"
      },
      "tagline" : "You Know, for Search"
    }
