## Mongodb集群
    Mongodb的replication主要有两种：主从和副本集（replica set）。主从的原理和mysql类似，主节点记录在其上的所有操作oplog，
    从节点定期轮询主节点获取这些操作，然后对自己的数据副本执行这些操作，从而保证从节点的数据与主节点一致。 
    现在mongodb官方建议用副本集替代主从复制
    
    主从集群和副本集最大的区别就是副本集没有固定的“主节点”；整个集群会选出一个“主节点”，
    当其挂掉后，又在剩下的从节点中选中其他节点为“主节点”，副本集总有一个活跃点(primary)和一个或多个备份节点(secondary)
    
    MongoDB分片的基本思想就是将集合切分成小块。这些块分散到若干片里面，每个片只负责总数据的一部分。应用程序不必知道哪片对应哪些数据，甚至不需要知道数据已经被拆分了，所以在分片之前要运行一个路由进程，该进程名为mongos
### StatefulSet定义
```shell script
apiVersion: v1
kind: ServiceAccount
metadata:
  name: mongo
  namespace: default
---

kind: ClusterRole
apiVersion: rbac.authorization.k8s.io/v1beta1
metadata:
  name: mongo
rules:
- apiGroups:
  - ""
  resources:
  - pods
  - persistentvolumes
  - persistentvolumeclaims
  verbs:
  - get
  - list
  - watch
  - update
  - create
  - delete
- apiGroups:
  - ""
  resources:
  - services
  - endpoints
  verbs:
  - get
  - list
  - watch
- apiGroups:
  - "storage.k8s.io"
  resources:
  - storageclasses
  verbs:
  - get
  - list
  - watch
- apiGroups:
  - ""
  resources:
  - secrets
  verbs:
  - get
  - list
  - watch
---

kind: ClusterRoleBinding
apiVersion: rbac.authorization.k8s.io/v1beta1
metadata:
  name: mongo
subjects:
- kind: ServiceAccount
  name: mongo
  namespace: default
roleRef:
  kind: ClusterRole
  name: mongo
  apiGroup: rbac.authorization.k8s.io
---

apiVersion: v1
kind: Service
metadata:
  name: mongo
  labels:
    name: mongo
spec:
  ports:
  - port: 27017
    targetPort: 27017
  clusterIP: None
  selector:
    role: mongo

---

apiVersion: apps/v1
kind: StatefulSet
metadata:
  name: mongo
spec:
  serviceName: "mongo"
  replicas: 3
  template:
    metadata:
      labels:
        role: mongo
        environment: test
    spec:
      terminationGracePeriodSeconds: 10
      containers:
        - name: mongo
          image: mongo:4
          command:
            - mongod
            - "--replSet"
            - rs0
            - "--bind_ip"
            - 0.0.0.0
            - "--smallfiles"
            - "--noprealloc"
          ports:
            - containerPort: 27017
          volumeMounts:
            - name: mongo-persistent-storage
              mountPath: /data/db
        - name: mongo-sidecar
          image: cvallance/mongo-k8s-sidecar
          env:
            - name: MONGO_SIDECAR_POD_LABELS
              value: "role=mongo,environment=test"
  volumeClaimTemplates:
  - metadata:
      name: mongo-persistent-storage
      annotations:
        volume.beta.kubernetes.io/storage-class: "nfs"
    spec:
      accessModes: [ "ReadWriteOnce" ]
      resources:
        requests:
          storage: 10Gi

```

### 验证
```shell script
#登录任意mongo pod
kubectl exec -it mongo-0 -- mongo
```

### statefulset 创建mongo集群, 集群外部访问
1. k8s是多节点集群,外部访问mongo集群NodeIP+NodePort即可。
    由于mongo的pod副本在不同节点上,通过service的nodeport暴露的mongo服务
2. k8s是单节点,外部访问mongo集群服务,只能将每个pod通过不同的service映射不同的端口,如:
```shell script
kind: Service
apiVersion: v1
metadata:
  name: mongo-0
spec:
  type: NodePort
  ports:
  - port: 27017
    targetPort: 27017
    protocol: TCP
    nodePort: 32624
  selector:
    statefulset.kubernetes.io/pod-name: mongo-0
---

kind: Service
apiVersion: v1
metadata:
  name: mongo-1
spec:
  type: NodePort
  ports:
  - port: 27017
    targetPort: 27017
    protocol: TCP
    nodePort: 32625
  selector:
    statefulset.kubernetes.io/pod-name: mongo-1
---

```

参考资料:
mongodb集群原理 https://blog.51cto.com/arthur376/2165039
mongodb 集群搭建的三种方式: https://blog.csdn.net/luonanqin/article/details/8497860
