
# GlusterFS
![架构](../../images/GlusterFS架构图.png)

## 介绍
    GlusterFS是Scale-Out存储解决方案Gluster的核心，它是一个开源的分布式文件系统，具有强大的横向扩展能力，通过扩展能够支持数PB存储容量和处理数千客户端。GlusterFS借助TCP/IP或InfiniBand RDMA网络将物理分布的存储资源聚集在一起，使用单一全局命名空间来管理数据。
    GlusterFS基于可堆叠的用户空间设计，可为各种不同的数据负载提供优异的性能。
    
    #常用术语
    Brick  最基本的存储单元，表示为trusted storage pool中输出的目录，供客户端挂载用
    Volume 一个卷，在逻辑上由N个bricks组成
    FUSE   Unix-like OS上的可动态加载的模块，允许用户不用修改内核即可创建自己的文件系统
    Glusterd Gluster management daemon，要在trusted storage pool中所有的服务器上运行
    POSIX  一个标准，GlusterFS兼容
    
    #优势
    1.无元数据服务设计，弹性HASH
    2.高性能：PB级容量、GB级吞吐量、数百集群规模
    3.用户空间模块化堆栈式设计
    4.高可用性，支持复制和自修复
    5.适合大文件存储
    
    #劣势
    1.大量小文件性能表现不佳
    2.复制存储利用率低

## 安装glusterfs
    # 先安装 gluster 源
    $ yum install centos-release-gluster -y
    # 安装 glusterfs 组件
    $ yum install -y glusterfs glusterfs-server glusterfs-fuse glusterfs-rdma glusterfs-geo-replication glusterfs-devel
    ## 创建 glusterfs 目录
    $ mkdir /opt/glusterd
    ## 修改 glusterd 目录
    $ sed -i 's/var\/lib/opt/g' /etc/glusterfs/glusterd.vol
    # 启动 glusterfs
    $ systemctl start glusterd.service
    # 设置开机启动
    $ systemctl enable glusterd.service
    #查看状态
    $ systemctl status glusterd.service

## 配置 glusterfs
    # 配置 hosts
    vi /etc/hosts
    172.16.1.228   gfs1
    172.16.1.244   gfs2
    172.16.1.248   gfs3
    
    # 开放端口
    $ iptables -I INPUT -p tcp --dport 24007 -j ACCEPT
    
    # 创建存储目录
    $ mkdir /opt/gfs_data
    
    # 添加节点到 集群
    # 执行操作的本机不需要probe 本机
    gluster peer probe gfs2
    gluster peer probe gfs3
    
    # 查看集群状态
    $ gluster peer status
## 配置 volume
    # 创建分布卷
    $ gluster volume create k8s-volume transport tcp gfs1:/opt/gfs_data gfs2:/opt/gfs_data gfs3:/opt/gfs_data force
    
    # 查看volume状态
    $ gluster volume info
    
    # 启动 分布卷
    $ gluster volume start k8s-volume

## Glusterfs调优
    # 开启 指定 volume 的配额
    $ gluster volume quota k8s-volume enable
    
    # 限制 指定 volume 的配额
    $ gluster volume quota k8s-volume limit-usage / 1TB
    
    # 设置 cache 大小, 默认32MB
    $ gluster volume set k8s-volume performance.cache-size 4GB
    
    # 设置 io 线程, 太大会导致进程崩溃
    $ gluster volume set k8s-volume performance.io-thread-count 16
    
    # 设置 网络检测时间, 默认42s
    $ gluster volume set k8s-volume network.ping-timeout 10
    
    # 设置 写缓冲区的大小, 默认1M
    $ gluster volume set k8s-volume performance.write-behind-window-size 1024MB